import liara


class GetProject:

    def get_project(self: "liara.Client", name: str):
        """
        Get details of a project
        :return:
        """
        path = "projects/" + name
        return self.api.execute(service=path, method="GET")




class APIError(Exception):

    def __init__(self, message, code=None):
        self.message = message
        self.code = code
        super().__init__(self.message)

    def __str__(self):
        if self.code is not None:
            return f"APIError {self.code}: {self.message}"
        else:
            return f"APIError: {self.message}"



from typing import Optional

import liara


class CreateApp:

    def create_app(
            self: "liara.Client",
            name: str,
            plan_id: str,
            platform: Optional[str] = None,
            network: Optional[str] = None
    ):
        """
        Create app that user owns
        :param plan_id: "enterprise", "standard", "free"
        """
        params: dict = {
            "name": name,
            "planID": plan_id,
            "platform": platform,
            "network": network
        }
        return self.api.execute(service="projects", method="POST", data=params)



from liara.errors import APIError
from liara.types import Results

from typing import Optional
import requests


class API:

    BASE_URL = 'https://api.iran.liara.ir/v1/'

    def __init__(self, client=None):
        self.client = client
        self.headers: dict = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Encoding': 'gzip, deflate, br',
            'Accept-Language': 'en-IN,en-GB;q=0.9,en-US;q=0.8,en;q=0.7,fa;q=0.6',
            'Authorization': f'Bearer {self.client.api_token}',
            'Origin': 'https://console.liara.ir',
            'Referer': 'https://console.liara.ir/',
            'User-Agent': (
                'Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
                'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
            )
        }

    def execute(self, service: str, method: str, data: Optional[dict] = None):
        """
        Execute HTTP request to Liara API
        """
        path = self.BASE_URL + service
        with requests.request(method=method, url=path, headers=self.headers, data=data) as response:
            response_data = response.json()
            if response.status_code == requests.codes.ok:
                return Results(response_data)
            raise APIError(response_data, response.status_code)



import json


class Results:

    def __str__(self) -> str:
        return self.jsonify(indent=2)

    def __getattr__(self, name):
        return self.find_keys(keys=name)

    def __setitem__(self, key, value):
        self.original_update[key] = value

    def __getitem__(self, key):
        return self.original_update[key]

    def __lts__(self, update: list, *args, **kwargs):
        for index, element in enumerate(update):
            if isinstance(element, list):
                update[index] = self.__lts__(update=element)
            elif isinstance(element, dict):
                update[index] = Results(update=element)
            else:
                update[index] = element
        return update

    def __init__(self, update: dict, *args, **kwargs) -> None:
        self.client = update.get("client")
        self.original_update = update

    def jsonify(self, indent=None) -> str:
        result = self.original_update
        result["original_update"] = "dict{...}"
        return json.dumps(result, indent=indent, ensure_ascii=False, default=lambda value: str(value))

    def find_keys(self, keys, original_update=None, *args, **kwargs):
        if original_update is None:
            original_update = self.original_update

        if not isinstance(keys, list):
            keys = [keys]

        if isinstance(original_update, dict):
            for key in keys:
                try:
                    update = original_update[key]
                    if isinstance(update, dict):
                        update = Results(update=update)
                    elif isinstance(update, list):
                        update = self.__lts__(update=update)
                    return update
                except KeyError:
                    pass
            original_update = original_update.values()

        for value in original_update:
            if isinstance(value, (dict, list)):
                try:
                    return self.find_keys(keys=keys, original_update=value)
                except AttributeError:
                    return None

        return None

    @property
    def to_dict(self) -> dict:
        """
        Return the update as dict
        """
        return self.original_update

    @property
    def user(self):
        return self.find_keys("user")

    @property
    def id(self):
        return self.user.find_keys("_id")

    @property
    def fullname(self):
        return self.user.find_keys("fullname")

    @property
    def national_code(self):
        return self.user.find_keys("nationalCode")

    @property
    def email(self):
        return self.user.find_keys("email")

    @property
    def phone(self):
        return self.user.find_keys("phone")

    @property
    def joined_at(self):
        return self.user.find_keys("joined_at")

    @property
    def free_credit_time(self):
        return self.user.find_keys("freeCreditTime")

    @property
    def remaining_free_credit(self):
        return self.user.find_keys("remainingFreeCredit")

    @property
    def balance(self):
        return self.user.find_keys("balance")

    @property
    def has_password(self):
        return self.user.find_keys("hasPassword")

    @property
    def min_credit_amount(self):
        return self.user.find_keys("minCreditAmount")

    @property
    def is_manual_min_credit(self):
        return self.user.find_keys("isManualMinCredit")

    @property
    def email_verified_at(self):
        return self.user.find_keys("emailVerifiedAt")

    @property
    def phone_verified_at(self):
        return self.user.find_keys("phoneVerifiedAt")

    @property
    def account_type(self):
        return self.user.find_keys("accountType")

    @property
    def is_verified(self):
        return self.user.find_keys("isVerified")

    @property
    def storage(self):
        return self.user.find_keys("storage")

    @property
    def storage_namespace(self):
        return self.storage.find_keys("namespace")

    @property
    def storage_status(self):
        return self.storage.find_keys("status")

    @property
    def avatar(self):
        return self.user.find_keys("avatar")

    @property
    def legacy_network_feature(self):
        return self.user.find_keys("legacyNetworkFeature")

    @property
    def private_network_feature(self):
        return self.user.find_keys("privateNetworkFeature")

    @property
    def legacy_object_storage_feature(self):
        return self.user.find_keys("legacyObjectStorageFeature")

    @property
    def legacy_domains_feature(self):
        return self.user.find_keys("legacyDomainsFeature")

    @property
    def current_subscription_plan(self):
        return self.user.find_keys("currentSubscriptionPlan")

    @property
    def database_versions(self):
        return self.find_keys("databaseVersions")

    @property
    def default_database_versions(self):
        return self.find_keys("defaultDatabaseVersions")

    @property
    def plans(self):
        return self.find_keys("plans")



from typing import Optional
from os.path import exists
import json

from liara.http import API
from liara.methods import Methods


class Client(Methods):

    def __init__(
            self,
            name: Optional[str] = None,
            api_token: Optional[str] = None,
            timeout: Optional[int] = 20
    ):
        self.name = name
        self.api_token = api_token
        self.timeout = timeout
        self.suffix = ".session"
        self.api = API(client=self)

        if self.api_token is None:
            self.filename = self.name
            if not self.name.endswith(self.suffix):
                self.filename += self.suffix
                if exists(self.filename):
                    self.load(self.filename)
                else:
                    data = self.login(
                        email=input("Enter your email: "),
                        password=input("Enter your password (is displayed on the screen): ")
                    )
                    self.api_token = data.get("api_token")
                    self.dump(data)

        self.api = API(client=self)
        print(f"connect to the Liara API with `{self.name}` session")

    def load(self, file_name):
        with open(file_name, "r") as file:
            data = json.loads(file.read())
            self.api_token = data.get("api_token")

    def dump(self, session):
        with open(self.filename, "w") as file:
            json.dump(session, file, indent=4)



import liara


class GetProjects:

    def get_projects_info(self: "liara.Client"):
        """
        Get all details of all projects that user owns
        """
        return self.api.execute(service="projects", method="GET")



from .create_app import CreateApp
from .get_projects import GetProjects
from .get_project import GetProject
from .delete_app import DeleteApp


class Apps(CreateApp, GetProjects, GetProject, DeleteApp):
    pass



import liara


class Login:

    def login(self: "liara.Client", email: str, password: str):
        params = {
            "email": email,
            "password": password
        }
        return self.api.execute(service="login", method="POST", data=params)



import liara


class DeleteApp:

    def delete_app(self: "liara.Client", name: str):
        """
        Delete a app
        """
        return self.api.execute(service="projects/" + name, method="DELETE")



from setuptools import setup, find_packages


setup(
    name='liara-cloud',
    version='2.0',
    author='AmirAli Irvani',
    author_email='irvaniamirali@proton.me',
    description=(
        'Unofficial library for account management and services in Liara Cloud'
    ),
    keywords=['liara', 'liara-cloud'],
    long_description=open('README.md', encoding='UTF-8').read(),
    python_requires="~=3.7",
    long_description_content_type='text/markdown',
    url='https://github.com/irvaniamirali/liara',
    packages=find_packages(),
    exclude_package_data={
        '': ['*.pyc', '*__pycache__*']
    },
    install_requires=['requests'],
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ]
)
